@@ -94,9 +94,8 @@ int lvm_lv_stack_delete(void *);
 
 /* andread@linpro.no */
 int lvm_get_free_space_list(char*, struct disk_info_t*);
-void *reduce_disk_usage_size(struct disk_info_t*,
-			     struct diskspace_req_s[],
-			     double);
+void reduce_disk_usage_size(struct disk_info_t*, struct diskspace_req_s[],
+			    double);
 void *lvm_vg_stack_new(void);
 int lvm_vg_stack_isempty(void *stack);
 int lvm_vg_stack_push(void *stack, const char *vgname);
@@ -109,8 +108,9 @@ int lvm_vg_stack_delete(void *);
 #include <sys/types.h>
 int make_path(const char *pathname, mode_t mode);
 
-/* ext3 is not supported by libparted v1.4, nor v1.6. */
-#define DEFAULT_FS "ext2"
+/* ext3 is not supported by libparted v1.4, nor v1.6.  Using hack to
+   work around it. */
+#define DEFAULT_FS "ext3"
 
 /* from evaluator.c */
 double evaluate(const char *expression);
