@@ -56,10 +56,11 @@ lvm_isinstalled(void)
         return FALSE;
     }
 
-    /* Is /proc/lvm a directory? */
-    if ( 0 != stat("/proc/lvm", &statbuf) || ! S_ISDIR(statbuf.st_mode) )
+    /* Is /proc/lvm a directory, or device-mapper loaded ? */
+    if ((0 != system("grep -q \"[0-9] device-mapper\" /proc/misc" )) &&
+        (0 != stat("/proc/lvm", &statbuf) || ! S_ISDIR(statbuf.st_mode)))
     {
-        autopartkit_error(0, "Missing /proc/lvm/, no LVM support available.");
+        autopartkit_error(0, "Missing /proc/lvm/ and no device-mapper in /proc/nmisc, no LVM support available.");
         isinstalled = FALSE;
         return FALSE;
     }
@@ -85,18 +86,23 @@ vg_exists(const char *vgname)
     struct stat statbuf;
     char *devpath = NULL;
     int retval = FALSE;
+    char *command = NULL;
 
     if (NULL == vgname)
         return FALSE;
 
-    if ( -1 == asprintf(&devpath, "/proc/lvm/VGs/%s", vgname) )
-        autopartkit_error(0, "Unable to allocate string for vg '%s'", vgname);
-    else
-    {
-        if (0 == stat(devpath, &statbuf) && S_ISDIR(statbuf.st_mode) )
-	    retval = TRUE;
-        free(devpath);
+    /* If the VG is missing, text is only printed on stderr.  Throw
+       away this, to detect if the VG existed or not. */
+    asprintf(&command, "vgdisplay \"%s\" 2>/dev/null |grep -q \"%s\"",
+	     vgname, vgname);
+
+    if ( NULL == command ) {
+        autopartkit_error(0, "Unable to allocate string for command");
+	return retval;
     }
+    if (0 == system(command))
+	retval = TRUE;
+    free(command);
     return retval;
 }
 
@@ -294,7 +300,12 @@ lvm_get_free_space_list(char *vgname, struct disk_info_t *spaceinfo)
     memset(&(spaceinfo->geom), 0, sizeof(spaceinfo->geom));
     autopartkit_log(2, "  Locating free space on volumegroup %s\n", vgname);
 
-    asprintf(&command, "/sbin/vgdisplay -c %s 2>&1", vgname);
+    /*
+     * Redirect errors to /dev/null, to avoid lots of messages claiming
+     * "File descriptor [3,4,5,6,7] left open", making it hard to find the
+     * VG size.
+     */
+    asprintf(&command, "/sbin/vgdisplay -c %s 2>/dev/null", vgname);
     autopartkit_log(2, "  Running command: %s\n", command);
     vgdisplay = popen(command, "r");
     if (! vgdisplay ){
@@ -353,6 +364,7 @@ lvm_get_free_space_list(char *vgname, struct disk_info_t *spaceinfo)
 	free(command);
         return 0;
     }
+    pclose (vgdisplay);
     free(command);
     return 1;
 }
